# Copyright (C) 2009,2010  Xyne
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.


package Xyne::Arch::BauerbillConf;
use strict;
use warnings;

use Xyne::Common::Functions qw/get_config_dir str/;
use Xyne::Arch::PowerpillConf;

our $VERSION = '0.10';

our @ISA = ("Xyne::Arch::PowerpillConf");

our @OPTIONS = map {$_->{description}="[Powerpill option]\n".$_->{description};$_} @Xyne::Arch::PowerpillConf::OPTIONS;
push (@OPTIONS,(

  {
    'name' => 'Arch',
    'aliases' => ['--arch'],
    'type' => 'str',
    'parameter_name' => '<architecture>',
    'description' => 'Specify the architecture to use if autodetection fails.'
  },

  {
    'name' => 'Autobuild',
    'aliases' => ['--autobuild'],
    'type' => 'list',
    'parameter_name' => '[pkgnames]',
    'description' => "Automatically build the specified packages using ABS. This works even when the \"ABS\" option is not set."
  },

  {
    'name' => 'Autopatch',
    'aliases' => ['--autopatch'],
    'type' => 'list',
    'parameter_name' => '[pkgnames]',
    'description' => "Automatically apply patches in the PatchDir to PKGBUILDs and install files when trusting the PKGBUILDs. If the prompt appears, the patch is not applied and it is left to the user."
  },

  {
    'name' => 'AUR',
    'aliases' => ['--aur'],
    'type' => 'switch',
    'description' => 'Enable AUR support.'
  },

  {
    'name' => 'ABS',
    'aliases' => ['--abs'],
    'type' => 'switch',
    'description' => 'Enable ABS support.'
  },

  {
    'name' => 'BuildAll',
    'aliases' => ['--build-all'],
    'type' => 'switch',
    'description' => 'Build all repo packages using ABS, including dependencies. This will bypass the ABS build queue dialogue.'
  },

  {
    'name' => 'BuildAs',
    'aliases' => ['--build-as'],
    'type' => 'str',
    'parameter_name' => '<user name>',
    'description' => 'If Bauerbill is run with root privileges, it will drop them and assume the privileges of the given user when sourcing files and building packages.'
  },

  {
    'name' => 'BuildDir',
    'aliases' => ['--build-dir'],
    'type' => 'str',
    'parameter_name' => '<path>',
    'default' => '/tmp/bauerbill/build',
    'description' => 'Specify the build directory.'
  },

  {
    'name' => 'CacheBuilt',
    'aliases' => ['--cache-built'],
    'type' => 'switch',
    'description' => 'Move built packages to the cache. If PKGDEST is set in makepkg.conf, it will take precedence over this option.'
  },

  {
    'name' => 'CPAN',
    'aliases' => ['--cpan'],
    'type' => 'switch',
    'description' => 'Enable CPAN support. Read the CPAN section of the man page before using this option.'
  },

  {
    'name' => 'Editor',
    'aliases' => ['--editor'],
    'type' => 'str',
    'parameter_name' => '<path>',
    'description' => 'Select a text editor for editing PKGBUILDs and install files.'
  },

  {
    'name' => 'GetCategories',
    'aliases' => ['--get-categories'],
    'type' => 'switch',
    'description' => 'Retrieve the AUR category list to enable translation of category IDs to category names when displaying information.'
  },

  {
    'name' => 'Hackage',
    'aliases' => ['--hackage'],
    'type' => 'switch',
    'description' => 'Enable Hackage support. Read the Hackage section of the man page before using this option.'
  },

  {
    'name' => 'HackageCache',
    'aliases' => ['--hackage-cache'],
    'type' => 'str',
    'description' => 'Specify the cache directory where Hackage metadata files will be mirrored and where the Hackage database file will be created.',
    'parameter_name' => '<path>',
    'default' => '/tmp/bauerbill/hackage'
  },

  {
    'name' => 'KeepMakedeps',
    'aliases' => ['--keep-makedeps'],
    'type' => 'switch',
    'description' => 'Keep build dependencies (makedeps) after a successful build.'
  },

  {
    'name' => 'MakepkgArgs',
    'aliases' => ['--makepkg-args'],
    'type' => 'str',
    'description' => "Additional arguments to makepkg which will be used when building packages from source. Check the makepkg documentation for details.\nNote that makepkg is NEVER RUN AS ROOT so there is no reason to pass it arguments which would allow it to be run as root.",
    'parameter_name' => '[makepkg args]'
  },

  {
    'name' => 'MakepkgConfig',
    'aliases' => ['--makepkg-config'],
    'type' => 'str',
    'description' => 'Specify a different makepkg configuration file to use.',
    'parameter_name' => '<path>',
    'default' => '/etc/makepkg.conf'
  },

  {
    'name' => 'PacpanCache',
    'aliases' => ['--pacpan-cache'],
    'type' => 'str',
    'description' => 'Specify the pacpan cache where CPAN metadata files will be mirrored and where the CPAN database file will be created.',
    'parameter_name' => '<path>',
    'default' => '/tmp/bauerbill/pacpan'
  },

  {
    'name' => 'PatchDir',
    'aliases' => ['--patch-dir'],
    'type' => 'str',
    'parameter_name' => '<path>',
    'description' => 'The PatchDir should be a directory with the same nested hierarchy as the BuildDir. It will be checked for corresponding PKGBUILD and install file patches to apply before sourcing a given PKGBUILD.'
  },

  {
    'name' => 'PowerpillConf',
    'aliases' => ['--powerpill-conf'],
    'type' => 'str',
    'parameter_name' => '<path>',
    'recommend' => '/etc/powerpill.conf',
    'description' => 'Specify a path to a Powerpill configuration file which should be loaded before the Bauerbill configuration file. This lets the user keep all common options in a single configuration file and selectively override them in the Bauerbill configution file, which also contains Bauerbill-specific options. Make sure that you comment all settings in the Bauerbill configuration file that you do not want to override.'
  },

  {
    'name' => 'TaurballDir',
    'aliases' => ['--taurball-dir'],
    'type' => 'str',
    'parameter_name' => '<path>',
    'default' => '/tmp/bauerbill/taurballs',
    'description' => 'Specify a storage directory for ABS and AUR taurballs. Taurballs are the archives containing the PKGBUILDs and local source files required by makepkg to build the package (AUR tarball = taurball).'
  },

  {
    'name' => 'TrustABS',
    'aliases' => ['--trust-abs'],
    'type' => 'switch',
    'description' => 'If set, the user will not be prompted to inspect PKGBUILDs and install files for ABS packages. If you trust the binary packages created with those PKGBUILDs, you can probably trust the PKGBUILDs themselves.'
  },

  {
    'name' => 'TrustCPAN',
    'aliases' => ['--trust-cpan'],
    'type' => 'switch',
    'description' => 'If set, the user will not be prompted to inspect PKGBUILDs generated by Xyne::Arch::CPAN. Although these PKGBUILD are safe in the sense that they will never contain malicious code, they may sometime require editing to correct errors that arise due to absent or incorrect metadata on CPAN.'
  },

  {
    'name' => 'TrustHackage',
    'aliases' => ['--trust-hackage'],
    'type' => 'switch',
    'description' => 'If set, the user will not be prompted to inspect PKGBUILDs generated by cabal2arch.'
  },

  {
    'name' => 'TrustedUsers',
    'aliases' => ['--trusted-users'],
    'type' => 'list',
    'parameter_name' => '[trusted AUR users]',
    'description' => "This option can be used to specify \"trusted users\" on the AUR. The user will not be prompted to inspect PKGBUILDs and install files for packages maintained by these \"trusted users\". Despite the name, this option is not limited to actual Arch TUs, although they are probably a good candidates for this list, as well as your own user account.\nThis option is provided for convenience and completeness but it is inherently dangerous. Use it with caution."
  },

  {
    'name' => 'UseCbreak',
    'aliases' => ['--use-cbreak'],
    'type' => 'switch',
    'description' => 'Enable stty cbreak at the PKGBUILD inspection prompt. This enables the user to make a choice without pressing <return>, e.g. by typing "ok" instead of "ok<return>". Some users have reported issues with this so it has been changed to an option.'
  },

  {
    'name' => 'Viewer',
    'aliases' => ['--viewer'],
    'type' => 'str',
    'parameter_name' => '<path>',
    'description' => 'Select a viewer for displaying text output. This can be used at the PKGBUILD inspection prompt to view text instead of printing it to the console. The viewer must accept text piped to it on STDIN, e.g. "/bin/less".'
  },










  {
    'name' => '--blindly-trust-everything-when-building-packages-despite-the-inherent-danger',
    'type' => 'switch',
    'description' => "This option enables pacman's \"--noconfirm\" option to bypass the PKGBUILD and install file inspection prompt for all packages. This option is VERY DANGEROUS and IT IS NOT RECOMMENDED THAT YOU USE THIS. The only reason that this is included is because some users desperately want this functionality.\nThe author highly recommends using the options to trust ABS packages and specific AUR users instead of this option, which can be used to achieve essentially the same effect."
  },

  {
    'name' => '--devel',
    'type' => 'switch',
    'description' => "Include all developmental packages in the upgrade operation as though they had been specified on the command line. Use this to rebuild packages under version control (foo-svn, foo-cvs, foo-git, etc)."
  },

  {
    'name' => '--get-taurballs',
    'type' => 'switch',
    'description' => "Download taurballs for the requested packages to TaurballDir and exit. Use Pacman's \"-d\" option if you do not want to download dependency taurballs.\nIf you would like to download the sources to BuildDir as well, use Pacman's \"-w\" option instead of this."
  },

));


our $MANPAGE = {
  'header' => 'BAUERBILL 1 "January 2010"',
  'sections' => [
    ['NAME', 'bauerbill - powerpill with abs and aur support'],
    ['SYNOPSIS', 'bauerbill [options] [packages]'],
    ['DESCRIPTION', "Bauerbill is an extension of Powerpill which includes support for building packages from the ABS and the AUR. You should refer to Powerpill's documentation for pure Powerpill information and Pacman's documentation for basic operations.\n\nBriefly, you should use bauerbill just as you would use pacman for most operations, e.g. \"bauerbill -Syu\" or \"bauerbill -S [pkgs]\". Without additional options, bauerbill, like powerpill, will speed up package downloads via parallel and segmented downloads. Via the options you can also enable several other features, such as building ABS, AUR and CPAN packages from source. See the information below and in the powerpill man page for more information about available options and features."],

    ['ABS',"Bauerbill does not require the \"abs\" package nor does it use it if it is available. Instead it uses \$repo.abs.tar.gz archives when they are available for a given repo. This enables Bauerbill to build packages from repos that are not included in the official ABS tree provided that they include this archive in their databases."],

    ['AUR',"Bauerbill uses the RPC interface to the AUR to perform searches and retrieve package information. It caches retrieved data internally to avoid redundant connections to the server and launches parallel queries to speed up operation times whenever possible.\n\nThe AUR is open to the public and despite the efforts of those who maintain it, harmful code may get uploaded due to either malice or ineptitude. You must inspect each and every package that you wish to build from the AUR to ensure that it does not pose any risk."],

    ['CPAN',"Bauerbill can also query information and build pacman packages from CPAN using the backend provided by Xyne::Arch::CPAN. It will intelligently try to map requested packages to CPAN distributions and will create packages with full depends, optdepends, makedepends and provides arrays whenever the metadata is available. In most cases it will therefore provide full dependency resolution and it will build all target packages while tracking build dependencies for removal.\n\nOccasionally the metadata file on CPAN (META.yml) will be absent, incomplete or erroneous and bauerbill the build will fail. In such cases the user must either choose to edit the PKGBUILD at the prompt or contact the CPAN author to fix the problem upstream.\n\nWhen CPAN support is enabled, you can specify a CPAN package by its Pacman package name (e.g. perl-foo-bar) or its CPAN distribution name (e.g. Foo-Bar), You can also specify any module contained within a distribution the same way and it will map to the correct CPAN package (e.g. perl-foo-bar-baz, Foo::Bar::Baz).\n\nOccasionally you will find that Perl packages in the official repos have been improperly packaged (e.g. names which deviate from the standard naming guidelines, provides array which are incomplete). You may want to consider using \"pacpan\" to fix these errors in local packages while waiting for the package maintainer to update the package. Pacpan provides a frontend to various functions in the Xyne::Arch::CPAN module and can be used to inspect installed Perl packages for errors and suggest corrections. It can even patch the local database provides arrays based by directly detecting modules in a package.\n\nPlease report all package errors detected by pacpan to the packager. Open a ticked on the bug tracker for official packages."],

    ['Hackage',"Bauerbill can retrieve information about Haskell libraries and programs on Hackage. If \"cabal2arch\" is installed, it can also build packages from Hackage with full dependency resolution and build dependency tracking and removal.\n\nInstall cabal2arch from the AUR using bauerbill if you do not already have it."],

    ['PERMISSIONS AND PRIVILEGES', "The balance of privileges versus restrictions was taken into consideration when writing Bauerbill. As an extension to Powerpill, it is desirable to be able to run Bauerbill as root to download packages directly to the default cache and start pacman without an additional prompt. At the same time, packages should never be built as root and makepkg will complain if you try.\n\nThe author has tried to resolve this issue by selectively dropping privileges when they are too high and by checking for sudo when they are too low. If Bauerbill is run as root, it will be able to download package to the default cache directory but it will change its effective user and group ids when sourcing PKGBUILDs and building packages, but will reset them when installing build packages.\n\nIf Bauerbill is run without special privileges, it will check for sudo when invoking pacman operations that require root. Note that you will need to specify a writable cache directory in this case."],

    ['PKGBUILDS AND SECURITY', "One of the great strengths of pacman is also one of its great weaknesses. PKGBUILDs are pure bash code and they are sourced (read: run) by makepkg when building packages. While this offers great simplicity for actually building packages, it presents a plethora of other problems related to it, mostly related to security and metadata extraction. In particular, sourcing an untrusted PKGBUILD is very dangerous (and part of the reason why you should never build packages as root).\n\nBauerbill tries to limit the risk of extracting metadata from a PKGBUILD while remaining versatile enough to recognize \"clever\" packager tricks that can obfuscate the values of variables. When Bauerbill sources a PKGBUILD, it first checks the syntax of the file without running it by using bash's \"-n\" flag. If the syntax is valid, Bauerbill then wraps the entire PKGBUILD in a function and sources it. By wrapping it in a function, the code is not executed but it is parsed and included in the output of \"set\" in canonical form. This makes it much easier to parse with regular expressions. Bauerbill then walks through the variables in order, interpolating and saving values as it goes, which keeping track of effective local environments.\n\nThis enables Bauerbill to extract everything it needs without actually executing the code, at least in cases of relatively well-behaved PKGBUILDs (including split packages).\n\nOf course the parser is not perfect and there will be special cases in which it fails miserably. Hopefully this can be improved over time.\n\nNote that despite this only mitigates the dangers of extracting metadata. You are still fully exposed to all of the dangers of the PKGBUILD when building the package."],

    ['CONFIGURATION FILE', "Bauerbill will check for a configuration file named bauerbill.conf in \"\$XDG_CONFIG_HOME/powerpill\" if it is set, else in \"\$HOME/.config/powerpill\". If a configuration file is not found, it will default to \"/etc/bauerbill.conf\". See below to learn how to generate a template conguration file."],

    ['OPTIONS', "Bauerbill supports the following superset of Powerpill's arguments in addition to Pacman's arguments:\n"],
    ['AUTHORS', 'Xyne']
  ]
};



our $CONFIG_TEMPLATE = {
#  'header' => "Each section begins with a name in square brackets, e.g. \"[options]\". There are currently only two reserved sections, namely [options] and [PkgD]. Any other section will be interpreted as a repo and servers specified within that section will be appended to the server list for that repo.",
  'sections' => [
    ['options', ""],
#    ['PkgD', "This section can be used to specify PkgD servers that should be queried before regular servers."]
  ]
};



our $HELP_MESSAGE = {
  'header' => "USAGE\n\n    bauerbill [bauerbill options] [pacman options] [args]\n\n    See \"--pacman-help\" for help with Pacman's options.\n\n\nBAUERBILL OPTIONS\n\n",
  'footer' => "",
};




our @DEFAULT_PATHS = (
  &get_config_dir().'/powerpill/bauerbill.conf',
  '/etc/bauerbill.conf'
);


sub new {
  my ($proto,$args) = @_;
  my $class = ref($proto) || $proto;
  my $self  = {};
  bless ($self, $class);

  $self->declare_options(@OPTIONS);
  $self->{CLI} = {};
  $self->{MANPAGE} = $MANPAGE;
  $self->{CONFIG_TEMPLATE} = $CONFIG_TEMPLATE;
  $self->{HELP_MESSAGE} = $HELP_MESSAGE;

  if (defined($args->{CALLBACKS}))
    { $self->{CALLBACKS} = $args->{CALLBACKS}; }
  else
    { $self->{CALLBACKS} = $Xyne::Arch::OptionHandler::CALLBACKS; }


  if (defined($args->{PacmanConf}))
    { $self->{PacmanConf} = $args->{PacmanConf}; }
  else
    { $self->{PacmanConf} = Xyne::Arch::PacmanConf->new(); }

  $self->{PacmanConf}->load_config($args->{'pacman_config_path'}) if $args->{'pacman_autoload'};
  $self->load_config($args->{'config_path'}) if $args->{'autoload'};

  return $self;
}


sub load_config
{
  my ($self,$args) = @_;
  my $pp_fpath = $args->{'powerpill_conf'};
  my $fpath = $args->{'config'};
  if (not defined($fpath))
  {
    foreach my $p (@DEFAULT_PATHS)
    {
      if (-r $p)
      {
        $fpath = $p;
        last;
      }

    }
  }
  my $config = &Xyne::Common::OptionHandler::load_config($self,$fpath);
  $pp_fpath = $self->get_value('PowerpillConf') if not defined($pp_fpath) and defined($self->get_value('PowerpillConf'));
  $self->unload_config();

  my $pp_config;
  if (defined($pp_fpath))
  {
    $pp_config = &Xyne::Common::OptionHandler::load_config($self,$pp_fpath);
    $self->unload_config();
  }

  $config->{'sections'}->{'options'} = [] if not defined($config->{'sections'}->{'options'});
  $config->{'sections'}->{'options'} = [
    @{$pp_config->{'sections'}->{'options'}},
    @{$config->{'sections'}->{'options'}}
  ] if defined($pp_config->{'sections'}->{'options'});

  $self->unset_all_values();
  $self->parse_config_options($config);
  $self->{CONFIG} = $config;
  $self->reload_cli();
  return $config;
}


1;

__END__

=head1 NAME

Xyne::Arch::BauerbillConf - Bauerbill configuration file class.

=head1 SYNOPSIS

  use Xyne::Arch::BauerbillConf;

=head1 DESCRIPTION

A class to load and parse Bauerbill configuration files and
store Bauerbill settings, including arguments passed to it from
Bauerbill objects.

=head2 EXPORT

None by default.

=head1 AUTHOR

Xyne <lt>xyne at archlinux dot ca<gt>

=head1 COPYRIGHT AND LICENSE

Copyright (C) 2009,2010  Xyne

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

=cut
