# Copyright (C) 2009,2010  Xyne
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.


package Xyne::Arch::PacmanConf;
use strict;
use warnings;

use Cwd qw/abs_path/;
use Xyne::Common::Functions qw/join_paths/;
use Xyne::Common::OptionHandler;

our @ISA = ("Xyne::Common::OptionHandler");

our $VERSION = '0.06';

our $DEFAULT_CONFIG_PATH = '/etc/pacman.conf';

our @OPTIONS = (
  {
    'name' => 'CacheDir',
    'aliases' => [qw/--cachedir/],
    'type' => 'list',
    'default' => ['/var/cache/pacman/pkg']
  },

  {
    'name' => 'CleanMethod',
    'type' => 'str'
  },

  {
    'name' => 'DBPath',
    'aliases' => [qw/-b --dbpath /],
    'type' => 'str',
    'default' => 'var/lib/pacman'
  },

  {
    'name' => 'HoldPkg',
    'type' => 'list'
  },

  {
    'name' => 'IgnoreGroup',
    'aliases' => [qw/--ignoregroup/],
    'type' => 'list'
  },

  {
    'name' => 'IgnorePkg',
    'aliases' => [qw/--ignore/],
    'type' => 'list'
  },

  {
    'name' => 'Include',
    'type' => 'str'
  },

  {
    'name' => 'LogFile',
    'aliases' => [qw/--logfile/],
    'type' => 'str',
    'default' => 'var/log/pacman.log'
  },

  {
    'name' => 'NoExtract',
    'type' => 'list'
  },

  {
    'name' => 'NoPassiveFtp',
    'type' => 'switch'
  },

  {
    'name' => 'NoUpgrade',
    'type' => 'list'
  },

  {
    'name' => 'RootDir',
    'aliases' => [qw/-r --root/],
    'type' => 'str',
    'default' => '/'
  },

  {
    'name' => 'Server',
    'type' => 'str'
  },

  {
    'name' => 'ShowSize',
    'type' => 'switch'
  },

  {
    'name' => 'SyncFirst',
    'type' => 'list'
  },

  {
    'name' => 'TotalDownload',
    'type' => 'switch'
  },

  {
    'name' => 'UseDelta',
    'type' => 'switch'
  },

  {
    'name' => 'UseSyslog',
    'type' => 'switch'
  },

  {
    'name' => 'XferCommand',
    'type' => 'str'
  },










  {
    'name' => '-Q',
    'aliases' => [qw/--query/],
    'type' => 'switch',
  },

  {
    'name' => '-R',
    'aliases' => [qw/--remove/],
    'type' => 'switch',
  },

  {
    'name' => '-S',
    'aliases' => [qw/--sync/],
    'type' => 'switch',
  },

  {
    'name' => '-T',
    'aliases' => [qw/--deptest/],
    'type' => 'switch',
  },

  {
    'name' => '-U',
    'aliases' => [qw/--upgrade/],
    'type' => 'switch',
  },

  {
    'name' => '-V',
    'aliases' => [qw/--version/],
    'type' => 'switch',
  },

  {
    'name' => '-c',
    'aliases' => [qw/--changelog --cascade --clean/],
    'type' => 'switch',
  },

  {
    'name' => '-d',
    'aliases' => [qw/--nodeps --deps/],
    'type' => 'switch',
  },

  {
    'name' => '-e',
    'aliases' => [qw/--explicit/],
    'type' => 'switch',
  },

  {
    'name' => '-f',
    'aliases' => [qw/--force/],
    'type' => 'switch',
  },

  {
    'name' => '-g',
    'aliases' => [qw/--groups/],
    'type' => 'switch',
  },

  {
    'name' => '-h',
    'aliases' => [qw/--help/],
    'type' => 'switch',
  },

  {
    'name' => '-i',
    'aliases' => [qw/--info/],
    'type' => 'switch',
  },

  {
    'name' => '-k',
    'aliases' => [qw/--keep/],
    'type' => 'switch',
  },

  {
    'name' => '-l',
    'aliases' => [qw/--list/],
    'type' => 'switch',
  },

  {
    'name' => '-m',
    'aliases' => [qw/--foreign/],
    'type' => 'switch',
  },

  {
    'name' => '-n',
    'aliases' => [qw/--nosave/],
    'type' => 'switch',
  },

  {
    'name' => '-o',
    'aliases' => [qw/--owns/],
    'type' => 'str',
  },

  {
    'name' => '-p',
    'aliases' => [qw/--file --print-uris/],
    'type' => 'switch',
  },

  {
    'name' => '-q',
    'aliases' => [qw/--quiet/],
    'type' => 'switch',
  },

  {
    'name' => '-s',
    'aliases' => [qw/--search --recursive/],
    'type' => 'switch',
  },

  {
    'name' => '-t',
    'aliases' => [qw/--unrequired/],
    'type' => 'switch',
  },

  {
    'name' => '-u',
    'aliases' => [qw/--upgrades --unneeded --sysupgrade/],
    'type' => 'switch',
  },

  {
    'name' => '-v',
    'aliases' => [qw/--verbose/],
    'type' => 'switch',
  },

  {
    'name' => '-w',
    'aliases' => [qw/--downloadonly/],
    'type' => 'switch',
  },

  {
    'name' => '-y',
    'aliases' => [qw/--refresh/],
    'type' => 'switch',
  },











  {
    'name' => '--asdeps',
    'type' => 'switch',
  },

  {
    'name' => '--asexplicit',
    'type' => 'switch',
  },

  {
    'name' => '--config',
    'type' => 'str',
  },

  {
    'name' => '--debug',
    'type' => 'switch',
  },

  {
    'name' => '--needed',
    'type' => 'switch',
  },

  {
    'name' => '--noconfirm',
    'type' => 'switch',
  },

  {
    'name' => '--noprogressbar',
    'type' => 'switch',
  },

  {
    'name' => '--noscriptlet',
    'type' => 'switch',
  },

);














########## CONSTRUCTOR ##########

sub new {
  my ($proto,$args) = @_;
  my $class = ref($proto) || $proto;
  my $self  = {};
  bless ($self, $class);

  $self->declare_options(@OPTIONS);
  $self->{CLI} = {};


  $self->load_config($args->{'config_path'}) if $args->{'autoload'};

  return $self;
}











########## CONFIG ##########

sub load_config
{
  my ($self,$fpath) = @_;
  $fpath = $DEFAULT_CONFIG_PATH if not defined($fpath);
  return &Xyne::Common::OptionHandler::load_config($self,$fpath);
}


# "Quirky paths" are paths which remain relative while unset but become
# absolte when set.
sub get_quirky_path
{
  my ($self,$path) = @_;
  return undef if not $path;
  if ($self->value_is_set($path))
  {
    return abs_path($self->get_value($path));
  }
  else
  {
    return join_paths($self->get_value('RootDir'),$self->get_value($path));
  }
}

sub get_dbpath
{
  return &get_quirky_path($_[0],'DBPath');
}

sub get_logfile
{
  return &get_quirky_path($_[0],'LogFile');
}

sub get_cachedirs
{
  my ($self) = @_;
  return $self->get_value('CacheDir');
}

sub get_rootdir
{
  my ($self) = @_;
  return $self->get_value('RootDir');
}








########## CLI ##########


sub parse_cli_opts
{
  my ($self,@opts) = @_;
  foreach my $opt (@opts)
  {
    my ($type,$name,$value) = @$opt;
    if ($type eq 'list') { $self->append_values($name,split(',',$value)); }
    else { $self->set_value($name,$value); }
  }
}









########## REPOS ##########

sub get_repo_list
{
  my ($self) = @_;
  return grep {$_ ne 'options'} $self->get_config_section_list();
}

sub get_repo_servers
{
  my ($self,$repo) = @_;
  return () if not defined($repo);
  return grep {$_} map {$1 if $_=~m/^\s*Server\s*=\s*(\S+)/} $self->get_config_section($repo)
}

sub add_repo_servers
{
  my ($self,$repo,@urls) = @_;
  $self->append_config_section($repo, [map {"Server = $_"} @urls]) if defined($repo) and $repo ne 'options';
}


1;

__END__

=head1 NAME

Xyne::Arch::PacmanConf - Pacman configuration file class.

=head1 SYNOPSIS

  use Xyne::Arch::PacmanConf;

=head1 DESCRIPTION

A class to load and parse pacman configuration files and
store pacman settings, including arguments passed to it from
pacman objects.

=head2 EXPORT

None by default.

=head1 AUTHOR

Xyne <lt>xyne at archlinux dot ca<gt>

=head1 COPYRIGHT AND LICENSE

Copyright (C) 2009,2010  Xyne

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
